%--------------------------------------------------------------------------
% BAYESIAN COMPUTATION OF POSTERIOR CONFIDENCE BANDS
% LEE SMITH
%--------------------------------------------------------------------------


function [IRpoint, IRup_2, IRup_1, IRlo_1, IRlo_2, IRmed] = confid_int(Bcomp,VC_eps,cvec,IRhoriz,NMC,Y,CIperc,expl);

%--------------------------------------------------------------------------
%
% DESCRIPTION:
%
% This function computes the impulse response functions and confidence intervals
%
%--------------------------------------------------------------------------
%
% INPUTS:
%
% Bcomp:   matrix with structure of estimated reduced-form coefficients in 
%           companion form excluding all the deterministic terms. 
%           This includes the estimated parameters in the partition 
%           n x (n x (n x vlag)). The remaining partition is made up of a 
%           diagonalic identity matrix that pins down the lead-lag relation 
%           between variables. The size is (n x vlag) x (n x vlag).     
%
% VC_eps:  covariance matrix of reduced-form residuals, size n x n.
%
% cvec:    matrix with estimated constants; this returns the scalar 0 
%           if no constant is included in the model; 
%           otherwise, this is a matrix of size n x 1.
%
% IRhoriz: number of periods for which point impulse responses are computed
%
% NMC:     number of Monte Carlo simulations for the density of simulated
%           impulse responses
%
% Y:       Raw Data
%
% CIperc:  this is a 2 x 1 vector with lower and upper percentiles for 
%           the confidence itervals that are computed and, eventually,
%           plotted
%
% expl:	   accept/reject explosive draws (0 accept, 1 reject)
%
%--------------------------------------------------------------------------
%
% OUTPUT:
%
% IRpoint: point estimate of impulse responses 
%
% IRup:    upper bound of impulse responses 
%
% IRlo:    lower bound of impulse responses
%
% IRmed:   median of impulse response distribution
%
%%----------------------------------------------------------------------------------------------------------------------------------
%% BACK OUT THE SIZE OF VARIOUS OBJECTS OF INTEREST
[Tbig,Nbig] = size(Y);
iscon  		= sum(abs(cvec))>0;
Nbig   		= length(VC_eps);
Nbigcomp 	= length(Bcomp);
vlag   		= length(Bcomp)/Nbig;
BIG_X 		= zeros(Tbig-vlag,Nbig*vlag+iscon);
if iscon == 1
	BIG_X(:,1) 	= ones(Tbig-vlag,1);
end;
for lags = 1:1:vlag;
	lower_index_row		= vlag+1-lags;
	upper_index_row		= Tbig-lags;
	upper_index_col		= Nbig*lags+iscon;
	lower_index_col		= upper_index_col - Nbig + 1;
	BIG_X(:,lower_index_col:upper_index_col) = Y(lower_index_row:upper_index_row,:); 	
end;

BIG_Y = Y(vlag+1:Tbig,:);

allIRs 	 = zeros(IRhoriz+1, Nbig, Nbig, NMC);
IRup_1   = zeros(IRhoriz+1, Nbig, Nbig);
IRlo_1   = IRup_1;
IRup_2	 = IRup_1;
IRlo_2	 = IRup_1;
IRmed  	 = IRup_1;

IRpoint=VAR_irf(Bcomp, VC_eps, IRhoriz);

l_CI = length(CIperc);
%%--------------------------------------------------------------
BIG_A 			= (BIG_X'*BIG_X)\BIG_X'*BIG_Y; 
vec_BIG_A 		= reshape(BIG_A,Nbig*(iscon+Nbig*vlag),1);
wishdof 		= Tbig;
% VC_eps is calcualted using MATLAB's 
% cov() function which divides the SSE by (Tbig-1).
% Therefore, 'scale=Tbig-1' undoes this so that we 
% are working with the raw SSE
scale			= Tbig-1;			
COVAR 			= inv(BIG_X'*BIG_X);
[ans,DI]		= iwishrnd(scale*VC_eps,wishdof); % THIS SPEEDS-UP DRAWS
% Tolerance for eigenvalue calculation
opts.tol = 1e-2; 

%% DO MC RUNS
for mm=1:NMC;
	draw = mm;
	workbar(mm/NMC,'Performing Monte Carlo Integration...','Progress');
	%% HERE I EMPLOY ANTITHETIC SAMPLING WHICH SPEEDS UP THE CONVERGENCE
	if mod(mm,2) == 1
		draw_again = 1;
		while draw_again == 1;
			% odd draw again
			VC_eps_draw 		= iwishrnd(scale*VC_eps,wishdof,DI);
			vec_BIG_A_COVAR		= kron(VC_eps_draw,COVAR);
			vec_A_draw_u		= mvnrnd(zeros(Nbig*(1+Nbig*vlag),1)',vec_BIG_A_COVAR);
			vec_BIG_A_draw		= vec_BIG_A + vec_A_draw_u'; 
			if expl == 1
				BIG_A_draw			= reshape(vec_BIG_A_draw,1+Nbig*vlag,Nbig);
				Bcomp_draw_1		=[BIG_A_draw(2:end,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
										zeros((vlag-1)*Nbig, Nbig)];
				e_1					= eigs(Bcomp_draw_1,1,'lm',opts);
				norm_1				= norm(e_1);
				vec_BIG_A_draw_2	= vec_BIG_A - vec_A_draw_u'; 
				BIG_A_draw_2		= reshape(vec_BIG_A_draw_2,1+Nbig*vlag,Nbig);
				Bcomp_draw_2		=[BIG_A_draw_2(2:end,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
										zeros((vlag-1)*Nbig, Nbig)];
				e_2					= eigs(Bcomp_draw_2,1,'lm',opts);
				norm_2				= norm(e_2);
				if norm_1 < 1.0+opts.tol && norm_2 < 1.0+opts.tol;
					draw_again = 0;
				end;
			elseif expl == 0
				draw_again = 0;
			end;
		end
	else
		% even draw - take reflection
		vec_BIG_A_draw		= vec_BIG_A - vec_A_draw_u'; 
	end;
	%%
	BIG_A_draw			= reshape(vec_BIG_A_draw,iscon+Nbig*vlag,Nbig);
	Bcomp_draw			=[BIG_A_draw(1+iscon:end,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
									zeros((vlag-1)*Nbig, Nbig)];
	allIRs(:, :, :, mm) = VAR_irf(Bcomp_draw, VC_eps_draw, IRhoriz);   
end
%% GET LOWER, MEDIAN AND UPPER VARIABLES OF INTEREST FROM SIMULATED DISTRIBUTION
%% IRFS
for hh=1:IRhoriz+1;
    for ii=1:Nbig;
        for jj=1:Nbig;
			IRpoint(hh,ii,jj) = IRpoint(hh,ii,jj);
            sliceloc   =  squeeze(allIRs(hh,ii,jj,:));
			if l_CI == 2;
            IRpercsloc = quantile(sliceloc,[CIperc(1);0.5;CIperc(2)]);
            IRlo_2(hh,ii,jj)   = IRpercsloc(1);
			IRmed(hh,ii,jj)    = IRpercsloc(2);
            IRup_2(hh,ii,jj)   = IRpercsloc(3);
			elseif l_CI == 4;
			IRpercsloc = quantile(sliceloc,[CIperc(1);CIperc(2);0.5;CIperc(3);CIperc(4)]);
            IRlo_1(hh,ii,jj)   = IRpercsloc(1);
			IRlo_2(hh,ii,jj)   = IRpercsloc(2);
            IRmed(hh,ii,jj)    = IRpercsloc(3);
			IRup_2(hh,ii,jj)   = IRpercsloc(4);
			IRup_1(hh,ii,jj)   = IRpercsloc(5);
			end;
        end
    end
end