%% THIS SCRIPT SIMULATES DATA FROM A NEW-KEYNESIAN DSGE MODEL 
%% UNDER WITH NOISE IN THE OBSERVED SURVEY FORECAST OF INTEREST RATES
%% THEN ESTIMATES A VAR AND IDENTIFIES FORWARD GUIDANCE SHOCKS 
%% USING SIGN RESTRICTIONS AND THE FORECAST CONSISTENT PRIOR
%% TRUE AND SVAR-IDENTIFIED IRFS ARE CALCULATED

clear;
close all;

% ADD VAR AUXILIARY FILES AND DYNARE TO PATH
addpath(strjoin({pwd, 'VARAuxiliaryFiles'},'\'));
addpath 'C:\dynare\4.5.7\matlab';

%% ADAPTIVE SURVEY EXPECTATIONS: 1>rho_exp>0
rho_exp_val	= 0.86; %% This is the weight on "old" information, from CG 2012 (JPE), they estimate this to be 0.86 using SPF data.

cd originalMODfiles/
save parameterfile rho_exp_val;
dynare NKModel_NoisyForecasts noclearall nolog
[result,eigenvalue_modulo,A,B,C,D] = ABCD_test(M_,options_,oo_);
cd ..

%% DEFINE VAR DATA
Y1			 = [x, pi, i, expi1];

%% SPECIFY LOCATION OF SURVEY FORECAST AND REALIZED DATA
s_loc	= 4;	% Location of survey measure in VAR
d_loc	= 3;	% Location of data measure in VAR

%% ESTIMATE VAR WITH CONSTANT
iscon = 1;

%% SET LAG LENGTH
[aic,bic,hqc] = get_lag_crit(Y1,iscon,24);
[~,vlag]=min(aic);

%% SET NUMBER OF IRF PERIODS (adds first period so 15 implies 16 periods will be computed)
IRhoriz = 23+(1+1);

%% SET OPTIONS FOR SIGN RESTRICTIONS
% Restrictions: positive 1, negative -1, unrestricted 0
R(:,:,1) = [ 0     0     0    % Real GDP
			 1     6    -1    % Inflation
			 0     0     0    % Fed Fund
			 1     6     1];  % 'Survey' Forecast of 12-Period Ahead Interest Rate

%% INITIALIZE VAR SAMPLE
sample_size  = 50100;
sample_start = 100;

%% DEFINE VAR SAMPLE 
sample_end 	 		= sample_start+sample_size;
Y 					= Y1(sample_start:sample_end,:);
dsge_fg_shocks  	= oo_.exo_simul(sample_start:sample_end,1); %fg_shock(sample_start:sample_end,1);%oo_.exo_simul(sample_start:sample_end,1); %fg_shock(sample_start+12:sample_end+12,1);
dsge_demand_shocks	= a_shock(sample_start:sample_end,1);
dsge_supply_shocks	= u_shock(sample_start:sample_end,1);
dsge_noise_shocks	= n_shock(sample_start:sample_end,1);
shocks_Xmat			= [normalize(dsge_fg_shocks), normalize(dsge_demand_shocks), normalize(dsge_supply_shocks), normalize(dsge_noise_shocks)];

corr_ssl = [];
for lfg = -24:1:24
	corr_ssl(lfg+25,:) = [lfg,corr(dsge_fg_shocks(1+24:end-24),dsge_fg_shocks(1+24+lfg:end-24+lfg))];
end
corr_ssl;

%% ESTIMATE VAR
[Bcomp, cvec, Bpl_ev, VC_eps, Residmat] = ...
								estim(Y, vlag, iscon);

%% TEST FOR INVERTIBILITY (R2 Should Be close to 1 if invertibility holds)
R2 = get_shock_R2(Y, vlag, iscon, dsge_fg_shocks);

%% SPECIFY VAR PRIOR
% Uninformative priors
[Tbig, Nbig] = size(Y);
A_prior = zeros(iscon+Nbig*vlag,Nbig);   
a_prior = A_prior(:);
V_prior = 1e12*eye(iscon+Nbig*vlag);
% Hyperparameters on inv(SIGMA) ~ W(v_prior,inv(S_prior))
v_prior = 0;
scale1  = Tbig-vlag-1;	
S_prior = 0*scale1*eye(Nbig);

%% SET OPTIONS FOR POSTERIOR SIMULATION
NMCDraws	  = 0;
NUhligDraws   = 200000;
NTargetAccept = 5000;
CIperc		  = [0.16,0.84];
[allIRs, allSResids, allFactors, allSs, allSigns] = MCMCUhligDraws(Bcomp,VC_eps,cvec,R,IRhoriz,CIperc,Y,NMCDraws,NUhligDraws,NTargetAccept,A_prior,a_prior,V_prior,v_prior,S_prior);

%% SET LAMBDA = 0 FOR "PURE SIGN RESTRICTIONS"
lambda = 0;
[allIRs_im0, allSResids_im0, mm_best0, mm_worst0, index_im0, effective_sample0] = WeakConsistencyPrior1period(allIRs,allSResids,lambda,s_loc,d_loc);

%% DEFINE PERIOD TO PLOT
IRhoriz = size(allIRs,1);	% Length of IRF
irflength = IRhoriz-13;	% Length of IRF to plot
t = 1:irflength;

%% GET LOWER, MEDIAN AND UPPER VARIABLES OF INTEREST FROM SIMULATED DISTRIBUTION
%% IRFS
for hh=1:size(allIRs_im0,1);
    for ii=1:size(allIRs_im0,2);
        for jj=1:size(allIRs_im0,3);
            sliceloc   =  squeeze(allIRs_im0(hh,ii,jj,:));
            IRpercsloc = quantile(sliceloc,[CIperc(1);0.5;CIperc(2)]);
            IRlo(hh,ii,jj)   = IRpercsloc(1);
			IRmed_im0(hh,ii,jj)  = IRpercsloc(2);
            IRup(hh,ii,jj)   = IRpercsloc(3);
        end
    end
end

%% SET LAMBDA > 0 FOR SIGN RESTRICTIONS + FORECAST CONSISTENT PRIOR
lambda = 5e13;
[allIRs_im, allSResids_im, mm_best, mm_worst, index_im, effective_sample] = WeakConsistencyPrior1period(allIRs,allSResids,lambda,s_loc,d_loc);

%% GET LOWER, MEDIAN AND UPPER VARIABLES OF INTEREST FROM SIMULATED DISTRIBUTION
%% IRFS
for hh=1:size(allIRs_im,1);
    for ii=1:size(allIRs_im,2);
        for jj=1:size(allIRs_im,3);
            sliceloc   =  squeeze(allIRs_im(hh,ii,jj,:));
            IRpercsloc = quantile(sliceloc,[CIperc(1);0.5;CIperc(2)]);
            IRlo(hh,ii,jj)   = IRpercsloc(1);
			IRmed_im(hh,ii,jj)  = IRpercsloc(2);
            IRup(hh,ii,jj)   = IRpercsloc(3);
        end
    end
end

SVAR_IRFS.im0  = IRmed_im0;
SVAR_IRFS.im   = IRmed_im;
SVAR_IRFS.dsge = [x_e_fg, pi_e_fg, i_e_fg, expi1_e_fg];

save('SVAR_IRFS','SVAR_IRFS');


