%% THIS SCRIPT SIMULATES DATA FROM A NEW-KEYNESIAN DSGE MODEL 
%% UNDER WITH NOISE IN THE OBSERVED SURVEY FORECAST OF INTEREST RATES
%% THEN ESTIMATES A VAR AND IDENTIFIES FORWARD GUIDANCE SHOCKS 
%% USING SIGN RESTRICTIONS AND THE FORECAST CONSISTENT PRIOR
%% AS WELL AS POINT IDENTIFICATION (VIA MINIMIZING FORECAST DISCREPANCY)
%% FOR EACH SVAR, WEIGHTS FOR THE 4 DSGE MODEL SHOCKS 
%% ARE ASSIGNED TO THE SVAR-IDENTIFIED FG SHOCK 

clear;
close all;

% ADD VAR AUXILIARY FILES AND DYNARE TO PATH
addpath(strjoin({pwd, 'VARAuxiliaryFiles'},'\'));
addpath 'C:\dynare\4.5.7\matlab';

%% ADAPTIVE SURVEY EXPECTATIONS: 1>rho_exp>0
rho_exp_val	= 0.86; %% This is the weight on "old" information, from CG 2012 (JPE), they estimate this to be 0.86 using SPF data.

cd originalMODfiles/
save parameterfile rho_exp_val;
dynare NKModel_NoisyForecasts noclearall nolog
[result,eigenvalue_modulo,A,B,C,D] = ABCD_test(M_,options_,oo_);
cd ..


%% DEFINE VAR DATA
Y1			 = [x, pi, i, expi1];

%% SPECIFY LOCATION OF SURVEY FORECAST AND REALIZED DATA
s_loc	= 4;	% Location of survey measure in VAR
d_loc	= 3;	% Location of data measure in VAR

%% ESTIMATE VAR WITH CONSTANT
iscon = 1;

%% SET LAG LENGTH
[aic,bic,hqc] = get_lag_crit(Y1,iscon,24);
[~,vlag]=min(aic);

%% SET NUMBER OF IRF PERIODS (adds first period so 15 implies 16 periods will be computed)
IRhoriz = 25+1;

%% SET OPTIONS FOR SIGN RESTRICTIONS
% Restrictions: positive 1, negative -1, unrestricted 0
R(:,:,1) = [ 0     0     0    % Real GDP
			 1     6    -1    % Inflation
			 0     0     0    % Fed Fund
			 1     6     1];  % 'Survey' Forecast of 12-Period Ahead Interest Rate

%% INITIALIZE VAR SAMPLE
sample_size  = 50100;
sample_start = 100;

%% DEFINE VAR SAMPLE 
sample_end 	 		= sample_start+sample_size;
Y 					= Y1(sample_start:sample_end,:);
dsge_fg_shocks  	= fg_shock(sample_start:sample_end,1); % oo_.exo_simul(sample_start:sample_end,1);
dsge_demand_shocks	= a_shock(sample_start:sample_end,1);
dsge_supply_shocks	= u_shock(sample_start:sample_end,1);
dsge_noise_shocks	= n_shock(sample_start:sample_end,1);
shocks_Xmat			= [normalize(dsge_fg_shocks), normalize(dsge_demand_shocks), normalize(dsge_supply_shocks), normalize(dsge_noise_shocks)];

dsge_fg_shocks_t_tau = fg_shock_t_1(sample_start:sample_end,1);

%% ESTIMATE VAR
[Bcomp, cvec, Bpl_ev, VC_eps, Residmat] = ...
								estim(Y, vlag, iscon);

%% TEST FOR INVERTIBILITY (R2 Should Be close to 1 if invertibility holds)
R2 = get_shock_R2(Y, vlag, iscon, dsge_fg_shocks);

%% SPECIFY VAR PRIOR
% Uninformative priors
[Tbig, Nbig] = size(Y);
A_prior = zeros(iscon+Nbig*vlag,Nbig);   
a_prior = A_prior(:);
V_prior = 1e12*eye(iscon+Nbig*vlag);
% Hyperparameters on inv(SIGMA) ~ W(v_prior,inv(S_prior))
v_prior = 0;
scale1  = Tbig-vlag-1;	
S_prior = 0*scale1*eye(Nbig);

%% SET OPTIONS FOR POSTERIOR SIMULATION
NMCDraws	  = 0;
NUhligDraws   = 200000;
NTargetAccept = 5000;
CIperc		  = [0.16,0.84];
[allIRs, allSResids, allFactors, allSs, allSigns] = MCMCUhligDraws(Bcomp,VC_eps,cvec,R,IRhoriz,CIperc,Y,NMCDraws,NUhligDraws,NTargetAccept,A_prior,a_prior,V_prior,v_prior,S_prior);

%% SET LAMBDA = 0 FOR "PURE SIGN RESTRICTIONS"
display(' ')
display('**********************')
display('Pure Sign Restrictions')
display('Identification: Set   ')
display('**********************')
display(' ')
lambda = 0;
[allIRs_im0, allSResids_im0, mm_best0, mm_worst0, index_im0, effective_sample0] = WeakConsistencyPrior1period(allIRs,allSResids,lambda,s_loc,d_loc);

% Calculate (median) forecast discrepancy
for dd=1:size(allSs,3);
	discrepancy_draws_im0(:,dd)  = SignAndWeakConsistencyCrit(allSs(1,:,dd),Bcomp,VC_eps,R,IRhoriz,s_loc,d_loc);
end

discrepancy_im0 = quantile(discrepancy_draws_im0,0.5);
fprintf(['Forecast discrepancy following an identified forward guidance shock: ',num2str(discrepancy_im0,'%.2f'),'\n']);

% Calculate (median) SVAR forward guidance shock and assocaited shock weights
for tt=1:size(allSResids_im0,1);
	sliceloc   				 = squeeze(allSResids_im0(tt,1,:)); 
	var_fg_shocks_sign(tt,1) = quantile(sliceloc,0.5);
end

norm_im0 = norm([corr(var_fg_shocks_sign,dsge_fg_shocks(vlag+1:end)), corr(var_fg_shocks_sign,dsge_demand_shocks(vlag+1:end)), corr(var_fg_shocks_sign,dsge_supply_shocks(vlag+1:end)), corr(var_fg_shocks_sign,dsge_noise_shocks(vlag+1:end))]);
norm_im0_t_tau = norm([corr(var_fg_shocks_sign,dsge_fg_shocks_t_tau(vlag+1:end)), corr(var_fg_shocks_sign,dsge_demand_shocks(vlag+1:end)), corr(var_fg_shocks_sign,dsge_supply_shocks(vlag+1:end)), corr(var_fg_shocks_sign,dsge_noise_shocks(vlag+1:end))]);

display('At Median of Pure Sign-Restrictions ')
display('Weight of SVAR-identified Forward Guidance Shock on: ')
fprintf(['DSGE Forward Guidance Shocks: ',num2str(corr(var_fg_shocks_sign,dsge_fg_shocks(vlag+1:end))/norm_im0,'%.2f'),'\n']);
fprintf(['DSGE Demand Shocks: 		   ',num2str(corr(var_fg_shocks_sign,dsge_demand_shocks(vlag+1:end))/norm_im0,'%.2f'),'\n']);
fprintf(['DSGE Supply Shocks: 		   ',num2str(corr(var_fg_shocks_sign,dsge_supply_shocks(vlag+1:end))/norm_im0,'%.2f'),'\n']);
fprintf(['DSGE Noise Shocks: 				',num2str(corr(var_fg_shocks_sign,dsge_noise_shocks(vlag+1:end))/norm_im0,'%.2f'),'\n']);
fprintf(['DSGE Forward Guidance Shocks (t-tau): ',num2str(corr(var_fg_shocks_sign,dsge_fg_shocks_t_tau(vlag+1:end))/norm_im0_t_tau,'%.2f'),'\n']);


%% SET LAMBDA > 0 FOR SIGN RESTRICTIONS + FORECAST CONSISTENT PRIOR
display(' ')
display('*******************************************')
display('Forecast Consistency with Sign Restrictions')
display('Identification: Set                        ')
display('*******************************************')
lambda = 5e13;
[allIRs_im, allSResids_im, mm_best, mm_worst, index_im, effective_sample] = WeakConsistencyPrior1period(allIRs,allSResids,lambda,s_loc,d_loc);

% Calculate (median) forecast discrepancy
allSs_im = allSs(:,:,index_im);

for dd=1:size(allSs_im,3);
	discrepancy_draws_im(:,dd)  = SignAndWeakConsistencyCrit(allSs_im(1,:,dd),Bcomp,VC_eps,R,IRhoriz,s_loc,d_loc);
end

discrepancy_im = quantile(discrepancy_draws_im,0.5);
fprintf(['Forecast discrepancy following an identified forward guidance shock: ',num2str(discrepancy_im,'%.2f'),'\n']);

% Calculate (median) SVAR forward guidance shock and assocaited shock weights
for tt=1:size(allSResids_im,1);
	sliceloc   				   = squeeze(allSResids_im(tt,1,:)); 
	var_fg_shocks_signfc(tt,1) = quantile(sliceloc,0.5);
end

norm_im = norm([corr(var_fg_shocks_signfc,dsge_fg_shocks(vlag+1:end)), corr(var_fg_shocks_signfc,dsge_demand_shocks(vlag+1:end)), corr(var_fg_shocks_signfc,dsge_supply_shocks(vlag+1:end)), corr(var_fg_shocks_signfc,dsge_noise_shocks(vlag+1:end))]);
norm_im_t_tau = norm([corr(var_fg_shocks_signfc,dsge_fg_shocks_t_tau(vlag+1:end)), corr(var_fg_shocks_signfc,dsge_demand_shocks(vlag+1:end)), corr(var_fg_shocks_signfc,dsge_supply_shocks(vlag+1:end)), corr(var_fg_shocks_signfc,dsge_noise_shocks(vlag+1:end))]);

display('At Median of Forecast Consistency with Sign Restrictions')
display('Weight of SVAR-identified Forward Guidance Shock on: ')
fprintf(['DSGE Forward Guidance Shocks: ',num2str(corr(var_fg_shocks_signfc,dsge_fg_shocks(vlag+1:end))/norm_im,'%.2f'),'\n']);
fprintf(['DSGE Demand Shocks: 			',num2str(corr(var_fg_shocks_signfc,dsge_demand_shocks(vlag+1:end))/norm_im,'%.2f'),'\n']);
fprintf(['DSGE Supply Shocks: 			',num2str(corr(var_fg_shocks_signfc,dsge_supply_shocks(vlag+1:end))/norm_im,'%.2f'),'\n']);
fprintf(['DSGE Noise Shocks: 			',num2str(corr(var_fg_shocks_signfc,dsge_noise_shocks(vlag+1:end))/norm_im,'%.2f'),'\n']);
fprintf(['DSGE Forward Guidance Shocks (t-tau): ',num2str(corr(var_fg_shocks_signfc,dsge_fg_shocks_t_tau(vlag+1:end))/norm_im_t_tau,'%.2f'),'\n']);

%% SEARCH OVER SET OF ROTATION MATRICES TO MINIMIZE FORECAST DISCREPANCY FOLLOWING A FORWARD GUIDNACE SHOCK	
display(' ')
display('***************************************************')
display('Minimum Forecast Discrepancy with Sign Restrictions')
display('Identification: Point                              ')
display('***************************************************')

%% Take "best" draw of rotation matrices as starting value
mm_guess = mm_best;
Q_guess  = allSs(:,:,mm_guess)';
q1guess = Q_guess(1,1);
q2guess = Q_guess(2,1);
q3guess = Q_guess(3,1);
q4guess = Q_guess(4,1);

%% Make sure everything is working ...
% Qguess_col1 = [q1guess,q2guess,q3guess,q4guess]';
% discrepancy_guess = SignAndWeakConsistencyCrit([q1guess,q2guess,q3guess,q4guess],Bcomp,VC_eps,R,IRhoriz,s_loc,d_loc);

% [Qguess_matrix,flag] = get_4d_orthonorm(q1guess,q2guess,q3guess,q4guess);
% Qguess_matrix(:,1);
% Factor_guess 		= chol(VC_eps)'*Qguess_matrix;
% inv_Factor_guess 	= inv(Factor_guess);
% SResids_guess 		= Residmat*inv_Factor_guess';

% var_fg_shocks_guess = SResids_guess(:,1);

% norm_guess = norm([corr(var_fg_shocks_guess,dsge_fg_shocks(vlag+1:end)), corr(var_fg_shocks_guess,dsge_demand_shocks(vlag+1:end)), corr(var_fg_shocks_guess,dsge_supply_shocks(vlag+1:end)), corr(var_fg_shocks_guess,dsge_noise_shocks(vlag+1:end))]);

% display('At Guess Values For Rotation Matrix ')
% display('Weight of SVAR-identified Forward Guidance Shock on: ')
% fprintf(['DSGE Forward Guidance Shocks: ',num2str(corr(var_fg_shocks_guess,dsge_fg_shocks(vlag+1:end))/norm_guess,'%.2f'),'\n']);
% fprintf(['DSGE Demand Shocks: 			',num2str(corr(var_fg_shocks_guess,dsge_demand_shocks(vlag+1:end))/norm_guess,'%.2f'),'\n']);
% fprintf(['DSGE Supply Shocks: 			',num2str(corr(var_fg_shocks_guess,dsge_supply_shocks(vlag+1:end))/norm_guess,'%.2f'),'\n']);
% fprintf(['DSGE Noise Shocks: 			',num2str(corr(var_fg_shocks_guess,dsge_noise_shocks(vlag+1:end))/norm_guess,'%.2f'),'\n']);

% fprintf(['Forecast discrepancy following an identified forward guidance shock: ',num2str(discrepancy_guess,'%.2f'),'\n']);

% Scale parameters in (-1,1) (a,b)
% Makes numerical optimization easier
[parametersguess] = get_scaledparams(q1guess,q2guess,q3guess,q4guess);
discrepancy = @(parameterestimation) SignAndWeakConsistencyCrit(parameterestimation,Bcomp,VC_eps,R,IRhoriz,s_loc,d_loc);
optionss=optimset('MaxFunEvals',1e12,'MaxIter',1e12,'TolX',1e-12,'TolFun',1e-12,'Display','final');
[Qstar_params,discrepancy_star] = fminsearch(discrepancy,parametersguess,optionss);

[q1star,q2star,q3star,q4star] = get_unscaledparams(Qstar_params);
[Qstar_matrix,flag] = get_4d_orthonorm(q1star,q2star,q3star,q4star);

Factor_star 	= chol(VC_eps)'*Qstar_matrix;
inv_Factor_star = inv(Factor_star);


%% Check Sign Restrictions
IR 			= VAR_irf_factor(Bcomp, Factor_star, IRhoriz);
Nbig	 = length(VC_eps);
Nshocks  = size(R,3);
checkall = ones(Nbig,Nshocks);
checkall_flip = ones(Nbig,Nshocks);

for ss=1:Nshocks
	for ii = 1:Nbig
		if R(ii,1,ss) ~= 0
			if R(ii,3,ss) == 1
				check = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) > 0;
				checkall(ii,ss) = min(check);
				% Check flipped signs
				check_flip = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) < 0;
				checkall_flip(ii,ss) = min(check_flip);
			elseif R(ii,3,ss) == -1
				check = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) < 0;
				checkall(ii,ss) = min(check);
				% Check flipped signs
				check_flip = IR((R(ii,1,ss)):R(ii,2,ss),ii,ss) > 0;
				checkall_flip(ii,ss) = min(check_flip);
			end
		end
	end
end


if min(min(checkall))==1
	SResids_star = Residmat*inv_Factor_star';
elseif min(min(checkall_flip))==1
	SResids_star = -1*Residmat*inv_Factor_star';
end

fprintf(['Forecast discrepancy following an identified forward guidance shock: ',num2str(discrepancy_star,'%.2f'),'\n']);

var_fg_shocks_star = SResids_star(:,1);

norm_star = norm([corr(var_fg_shocks_star,dsge_fg_shocks(vlag+1:end)), corr(var_fg_shocks_star,dsge_demand_shocks(vlag+1:end)), corr(var_fg_shocks_star,dsge_supply_shocks(vlag+1:end)), corr(var_fg_shocks_star,dsge_noise_shocks(vlag+1:end))]);
norm_star_t_tau = norm([corr(var_fg_shocks_star,dsge_fg_shocks_t_tau(vlag+1:end)), corr(var_fg_shocks_star,dsge_demand_shocks(vlag+1:end)), corr(var_fg_shocks_star,dsge_supply_shocks(vlag+1:end)), corr(var_fg_shocks_star,dsge_noise_shocks(vlag+1:end))]);

display('At Optimized Values For Rotation Matrix ')
display('Weight of SVAR-identified Forward Guidance Shock on: ')
fprintf(['DSGE Forward Guidance Shocks: ',num2str(corr(var_fg_shocks_star,dsge_fg_shocks(vlag+1:end))/norm_star,'%.2f'),'\n']);
fprintf(['DSGE Demand Shocks: 			',num2str(corr(var_fg_shocks_star,dsge_demand_shocks(vlag+1:end))/norm_star,'%.2f'),'\n']);
fprintf(['DSGE Supply Shocks: 			',num2str(corr(var_fg_shocks_star,dsge_supply_shocks(vlag+1:end))/norm_star,'%.2f'),'\n']);
fprintf(['DSGE Noise Shocks: 			',num2str(corr(var_fg_shocks_star,dsge_noise_shocks(vlag+1:end))/norm_star,'%.2f'),'\n']);
fprintf(['DSGE Forward Guidance Shocks (t-tau): ',num2str(corr(var_fg_shocks_star,dsge_fg_shocks_t_tau(vlag+1:end))/norm_star_t_tau,'%.2f'),'\n']);