%% THIS SCRIPT SIMULATES DATA FROM A NEW-KEYNESIAN DSGE MODEL 
%% UNDER WITH NOISE IN THE OBSERVED SURVEY FORECAST OF INTEREST RATES
%% THEN ESTIMATES A VAR AND IDENTIFIES FORWARD GUIDANCE SHOCKS 
%% USING SIGN RESTRICTIONS AND THE FORECAST CONSISTENT PRIOR
%% FOR EACH SVAR, WEIGHTS (AS DEFINED IN WOLF, 2020) FOR THE 4 DSGE MODEL SHOCKS 
%% ARE ASSIGNED TO THE SVAR-IDENTIFIED FG SHOCK 

%% THIS CODE AND SUPPORTING FILES ARE BASED ON THE REPLICATION CODE FOR:
%% Wolf, Christian K. (2020) SVAR (mis)identification and the real effects of monetary policy shocks.
%% AEJ: Macro

clear;
close all;

% ADD VAR AUXILIARY FILES AND DYNARE TO PATH
addpath(strjoin({pwd, 'VARAuxiliaryFiles'},'\'));
addpath(strjoin({pwd, 'Wolf2020_files'},'\'));
addpath 'C:\dynare\4.5.7\matlab';

%----------------------------------------------------------------
% IS Characterization
%----------------------------------------------------------------

settings.VMA_hor          = 350; % maximal horizon for VMA representation
settings.VAR_poplaglength = 250; % lag length in population VAR
settings.IRF_hor          = 30; % horizon for computed IRFs
settings.IS_draws         = 20000; % number of draws from the identified set
settings.weight_hor       = 20; % horizon for weight polynomial P(L)
settings.HP_smooth        = 1000000; % smoothing parameter for shock weights
settings.HP_target        = 1; % smoothing parameter for shock weights (irrelevant here)

%----------------------------------------------------------------
% Identifying Restrictions
%----------------------------------------------------------------
% (x,pi,i,expi1)

%% SPECIFY LOCATION OF SURVEY FORECAST AND REALIZED DATA
IS.s_loc			= 4; % Location of survey measure in VAR
IS.d_loc			= 3; % Location of data measure in VAR

IS.restrict_hor = 4;
IS.restrict_sign = [0 -1 0 1];
IS.restrict_mat = repmat(IS.restrict_sign,IS.restrict_hor,1);
IS.shock_pos = 1;

%% VAR REPRESENTATION

%----------------------------------------------------------------
% Solve DSGE Model Using Dynare
%----------------------------------------------------------------

% model run

%% ADAPTIVE SURVEY EXPECTATIONS: 1>rho_exp>0
rho_exp_val	= 0.86; %% This is the weight on "old" information, from CG 2012 (JPE), they estimate this to be 0.86 using SPF data.

cd originalMODfiles/
save parameterfile rho_exp_val;
dynare NKModel_NoisyForecasts noclearall nolog
cd ..

disp('I have solved and simulated the model.')

disp('Collecting model properties...')

% get law of motion for all model variables

decision = disp_dr_Wolf2020(oo_.dr,options_.order,[]);
DSGE_model.decision = decision(2:end,:);

% specify observables

DSGE_model.obs = options_.varobs_id; %[1 2 3 7]; % (x,pi,i,expi1)

% size indicators

DSGE_model.n_y   = size(DSGE_model.obs,2);
DSGE_model.n_eps = M_.exo_nbr; % 4 structural shocks
DSGE_model.n_s   = M_.nspred;

% ABCD representations

DSGE_model.ABCD = ABCD_fun(DSGE_model);

%----------------------------------------------------------------
% Get Population IRFs + FVDs + Shock Sequences
%----------------------------------------------------------------

[DSGE_model.IRF,DSGE_model.M,DSGE_model.tot_weights] = pop_analysis(DSGE_model,settings);

disp('...done!')

%----------------------------------------------------------------
% Get VAR Representation
%----------------------------------------------------------------

disp('Getting the VAR representation...')

VAR = popVAR(DSGE_model,settings);

disp('...done!')

%% GETTING THE IDENTIFIED SET

%----------------------------------------------------------------
% Benchmark Rotation
%----------------------------------------------------------------

IS.bench_rot = chol(VAR.Sigma_u,'lower');

%----------------------------------------------------------------
% Get Identified Set
%----------------------------------------------------------------

IS.W = NaN(DSGE_model.n_y,DSGE_model.n_y,settings.IS_draws);

for i_draws = 1:settings.IS_draws
    
    if mod(i_draws,100) == 0
        disp('Number of runs is:')
        disp(i_draws)
    end
    
    control = 0;
    
    while control == 0
       
        newmatrix=normrnd(0,1,DSGE_model.n_y,DSGE_model.n_y);
        [Q,R]=qr(newmatrix);
        for j = 1:DSGE_model.n_y
            if R(j,j) < 0
                Q(:,j) = -Q(:,j);
            end
        end
         
        W_cand = IS.bench_rot * Q;
        
        if W_cand(DSGE_model.n_y,IS.shock_pos) < 0
            W_cand(:,IS.shock_pos) = - W_cand(:,IS.shock_pos);
        end
        
        IRF_check = NaN(DSGE_model.n_y,DSGE_model.n_y,IS.restrict_hor);
        for i_hor = 1:IS.restrict_hor
            IRF_check(:,:,i_hor) = VAR.IRF_Wold(:,:,i_hor) * W_cand;
        end
        IRF_check = squeeze(IRF_check(:,IS.shock_pos,:))';
        
        if sum((IRF_check(:) .* IS.restrict_mat(:) >= 0)) == size(IS.restrict_mat,1) * size(IS.restrict_mat,2)
        else
            continue
        end
            
        control = 1;
        
    end
    
    IS.W(:,:,i_draws) = W_cand;
        
end

clear control i_draws IRF_check newmatrix Q R W_cand j l

%----------------------------------------------------------------
% Get IRFs
%----------------------------------------------------------------

IS.IRF = NaN(DSGE_model.n_y,DSGE_model.n_y,settings.IRF_hor,settings.IS_draws);
for i_draws = 1:settings.IS_draws
    for i_hor = 1:settings.IRF_hor
        IS.IRF(:,:,i_hor,i_draws) = VAR.IRF_Wold(:,:,i_hor) * IS.W(:,:,i_draws);
    end
end

IS.IRF = permute(squeeze(IS.IRF(:,IS.shock_pos,:,:)),[2 1 3]);


for i_hor = 1:settings.IRF_hor
    for i_y = 1:DSGE_model.n_y
        IS.IRF_med(i_hor,i_y) = quantile(IS.IRF(i_hor,i_y,:),0.5);
        IS.IRF_16(i_hor,i_y) = quantile(IS.IRF(i_hor,i_y,:),0.16);
        IS.IRF_84(i_hor,i_y) = quantile(IS.IRF(i_hor,i_y,:),0.84);
        IS.IRF_lb(i_hor,i_y) = min(IS.IRF(i_hor,i_y,:));
        IS.IRF_ub(i_hor,i_y) = max(IS.IRF(i_hor,i_y,:));
    end
end

[IS.y0_sorted,IS.y0_index] = sort(squeeze(IS.IRF(1,1,:)),'descend');

%----------------------------------------------------------------
% Shock Weights
%----------------------------------------------------------------

IS.weights = NaN(DSGE_model.n_y,DSGE_model.n_eps,settings.weight_hor,settings.IS_draws);
for i_draws = 1:settings.IS_draws
    for i_hor = 1:settings.weight_hor
        IS.weights(:,:,i_hor,i_draws) = IS.W(:,:,i_draws)^(-1) * DSGE_model.M(:,:,i_hor);
    end
end

IS.weights = squeeze(IS.weights(IS.shock_pos,:,1,:))';
IS.weights = IS.weights(IS.y0_index,:);

%----------------------------------------------------------------
% Shock Weights For Different Degrees of Forecast Consistency
%----------------------------------------------------------------

%% SET LAMBDA VALUES TO LOOP OVER
lambdas 	 		= 0:0.05e13:1e13;
SVAR_Wolf2020weights(:,:)   = NaN(length(lambdas),5);

for ll = 1:1:length(lambdas);
	
	[IRF_im, mm_best, mm_worst, index_im, effective_sample] = WeakConsistencyPrior1period_WolfCode(IS.IRF,lambdas(ll),IS.s_loc,IS.d_loc);
	
	W_im = IS.W(:,:,index_im);
	
	weights_im = NaN(DSGE_model.n_y,DSGE_model.n_eps,settings.weight_hor,settings.IS_draws);
	for i_draws = 1:settings.IS_draws
		for i_hor = 1:settings.weight_hor
			weights_im(:,:,i_hor,i_draws) = W_im(:,:,i_draws)^(-1) * DSGE_model.M(:,:,i_hor);
		end
	end

	weights_imf = squeeze(weights_im(IS.shock_pos,:,1,:))';
	
	fg_weight		= quantile(weights_imf(:,1),0.5);
	noise_weight	= quantile(weights_imf(:,2),0.5);
	demand_weight	= quantile(weights_imf(:,3),0.5);
	supply_weight	= quantile(weights_imf(:,4),0.5);
	
	SVAR_Wolf2020weights(ll,:) = [lambdas(ll),fg_weight,demand_weight,supply_weight,noise_weight];
	
	W_im = [];
	
end

save('SVAR_Wolf2020weights.mat','SVAR_Wolf2020weights');



