%% THIS SCRIPT SIMULATES DATA FROM A NEW-KEYNESIAN DSGE MODEL 
%% UNDER WITH NOISE IN THE OBSERVED SURVEY FORECAST OF INTEREST RATES
%% THEN ESTIMATES A VAR AND IDENTIFIES FORWARD GUIDANCE SHOCKS 
%% USING SIGN RESTRICTIONS AND THE FORECAST CONSISTENT PRIOR
%% FOR EACH SVAR, WEIGHTS FOR THE 4 DSGE MODEL SHOCKS 
%% ARE ASSIGNED TO THE SVAR-IDENTIFIED FG SHOCK 

clear;
close all;

% ADD VAR AUXILIARY FILES AND DYNARE TO PATH
addpath(strjoin({pwd, 'VARAuxiliaryFiles'},'\'));
addpath 'C:\dynare\4.5.7\matlab';

%% ADAPTIVE SURVEY EXPECTATIONS: 1>rho_exp>0
rho_exp_val	= 0.86; %% This is the weight on "old" information, from CG 2012 (JPE), they estimate this to be 0.86 using SPF data.

cd originalMODfiles/
save parameterfile rho_exp_val;
dynare NKModel_NoisyForecasts noclearall nolog
[result,eigenvalue_modulo,A,B,C,D] = ABCD_test(M_,options_,oo_);
cd ..

%% DEFINE VAR DATA
Y1			 = [x, pi, i, expi1];

%% SPECIFY LOCATION OF SURVEY FORECAST AND REALIZED DATA
s_loc	= 4;	% Location of survey measure in VAR
d_loc	= 3;	% Location of data measure in VAR

%% ESTIMATE VAR WITH CONSTANT
iscon = 1;

%% SET LAG LENGTH BASED ON AIC
[aic,bic,hqc] = get_lag_crit(Y1,iscon,24);
[~,vlag]=min(aic)

%% SET NUMBER OF IRF PERIODS, ADD TAU-PERIODS TO CALCULATE FORECAST DISCREPANCY
IRhoriz = 24+(1);

%% SET OPTIONS FOR SIGN RESTRICTIONS
% Restrictions: positive 1, negative -1, unrestricted 0
R(:,:,1) = [ 0     0     0    % Real GDP
			 1     6    -1    % Inflation
			 0     0     0    % Fed Fund
			 1     6     1];  % 'Survey' Forecast of Interest Rate

%% INITIALIZE VAR SAMPLE
sample_size  = 50100;
sample_start = 100;

%% DEFINE VAR SAMPLE 
sample_end 	 		= sample_start+sample_size;
Y 					= Y1(sample_start:sample_end,:);
dsge_fg_shocks  	= fg_shock(sample_start:sample_end,1); % oo_.exo_simul(sample_start:sample_end,1);
dsge_demand_shocks	= a_shock(sample_start:sample_end,1);
dsge_supply_shocks	= u_shock(sample_start:sample_end,1);
dsge_noise_shocks	= n_shock(sample_start:sample_end,1);
shocks_Xmat			= [normalize(dsge_fg_shocks), normalize(dsge_demand_shocks), normalize(dsge_supply_shocks), normalize(dsge_noise_shocks)];

%% ESTIMATE VAR
[Bcomp, cvec, Bpl_ev, VC_eps, Residmat] = ...
								estim(Y, vlag, iscon);

%% TEST FOR INVERTIBILITY (R2 Should Be close to 1 if invertibility holds)
R2 = get_shock_R2(Y, vlag, iscon, dsge_fg_shocks);

%% SPECIFY VAR PRIOR
% Uninformative priors
[Tbig, Nbig] = size(Y);
A_prior = zeros(iscon+Nbig*vlag,Nbig);   
a_prior = A_prior(:);
V_prior = 1e12*eye(iscon+Nbig*vlag);
% Hyperparameters on inv(SIGMA) ~ W(v_prior,inv(S_prior))
v_prior = 0;
scale1  = Tbig-vlag-1;	
S_prior = 0*scale1*eye(Nbig);

%% SET OPTIONS FOR POSTERIOR SIMULATION
NMCDraws	  = 0;
NUhligDraws   = 200000;
NTargetAccept = 5000;
CIperc		  = [0.16,0.84];
[allIRs, allSResids, allFactors, allSs, allSigns] = MCMCUhligDraws(Bcomp,VC_eps,cvec,R,IRhoriz,CIperc,Y,NMCDraws,NUhligDraws,NTargetAccept,A_prior,a_prior,V_prior,v_prior,S_prior);

%% SET LAMBDA VALUES TO LOOP OVER
lambdas 	 = 0:0.05e12:1e12;
lambdaslinsp = linspace(0.3,0.8,length(lambdas));
irflength 	 = IRhoriz-12;
Reg_Coeffs	 = NaN(length(lambdas),5);

for ll = 1:1:length(lambdas);

	[allIRs_im, allSResids_im, mm_best, mm_worst, index_im, effective_sample] = WeakConsistencyPrior1period(allIRs,allSResids,lambdas(ll),s_loc,d_loc);
	
	for dd = 1:size(allSResids_im,3);
		[allReg_Coeffs(:,dd),~, ~, ~, allStats(dd,:)] = regress(normalize(allSResids_im(:,1,dd)), [ones(size(shocks_Xmat(vlag+1:end,:),1),1), shocks_Xmat(vlag+1:end,:)]);
	end
	
	for ss = 1:5;
		Reg_Coeffs(ll,ss) = quantile(allReg_Coeffs(ss,:),0.5);
	end;
	
	R2_shocks_reg(ll,1)	= quantile(allStats(:,1),0.5);
	
	for tt=1:size(allSResids_im,1);
		sliceloc   		= squeeze(allSResids_im(tt,1,:)); 
		SSmed_im(tt,1) 	= quantile(sliceloc,0.5);
	end
	
	norm_im = norm([corr(SSmed_im,dsge_fg_shocks(vlag+1:end)), corr(SSmed_im,dsge_demand_shocks(vlag+1:end)), corr(SSmed_im,dsge_supply_shocks(vlag+1:end)), corr(SSmed_im,dsge_noise_shocks(vlag+1:end))]);
	
	SVAR_weights(ll,:)	= [lambdas(ll),corr(SSmed_im,dsge_fg_shocks(vlag+1:end))/norm_im,...
							   corr(SSmed_im,dsge_demand_shocks(vlag+1:end))/norm_im,...
							   corr(SSmed_im,dsge_supply_shocks(vlag+1:end))/norm_im,...
							   corr(SSmed_im,dsge_noise_shocks(vlag+1:end))/norm_im, effective_sample, ...
							   Reg_Coeffs(ll,2:5)];
end

save('SVAR_weights.mat','SVAR_weights');