%--------------------------------------------------------------------------
% BAYESIAN COMPUTATION OF POSTERIOR CONFIDENCE BANDS
% LEE SMITH
%--------------------------------------------------------------------------


function [IRpoint, IRup_2, IRup_1, IRlo_1, IRlo_2, IRmed] = confid_int(Bcomp,VC_eps,cvec,dvec,IRhoriz,IRtype,sizesho,NMC,Y,CIperc);

%--------------------------------------------------------------------------
% 
% DESCRIPTION:
%
% This function computes the impulse response functions and confidence intervals
%
%--------------------------------------------------------------------------
%
% INPUTS:
%
% Bcomp:   matrix with structure of estimated reduced-form coefficients in 
%           companion form excluding all the deterministic terms. 
%           This includes the estimated parameters in the partition 
%           n x (n x (n x vlag)). The remaining partition is made up of a 
%           diagonalic identity matrix that pins down the lead-lag relation 
%           between variables. The size is (n x vlag) x (n x vlag).     
%
% VC_eps:  covariance matrix of reduced-form residuals, size n x n.
%
% cvec:    matrix with estimated constants; this returns the scalar 0 
%           if no constant is included in the model; 
%           otherwise, this is a matrix of size n x 1.
%
% dvec:    matrix with estimated parameters on the deterministic trends;
%           this returns the scalar 0 if no constant is included 
%           otherwise, this is a matrix of size n x 1.
%
% IRhoriz: number of periods for which point impulse responses are computed
%
%
% IRtype:  this string variable can be assigned the values'c' or 'g'
%
% sizesho: this is a singleton that is used to normalize the Choleski 
%           transformation of the variance-covariance matrix of 
%           reduced-form residuals. If it has a value equal to 0, 
%           alternative normalizations of the Choleski are used,
%           according to the following table
%
%
% Table 1: structural shock vector as a function of the two key inputs
% ---------------------------------------------------------------------------------------------------------
% |            |                                                                                          |
% |            |              IRtype=c                                    IRtype=g                        |
% |            |------------------------------------------------------------------------------------------|
% |            |                                                                                          |
% | sizesho=0  |         VC_eps_chol*eye(Nbig)                    VC_eps*diag(stdepsvec.^(-1))            |
% |            |                                                                                          |
% |sizesho~=0  |   VC_eps_chol*diag(sizesho./stdepsvec)   VC_eps_chol*diag(stdepsvec.^(-2))*diag(sizesho) |
% |            |                                                                                          |
% |            |                                                                                          |
% ---------------------------------------------------------------------------------------------------------
% Notes: 
% A. VC_eps denotes the variance-covariance decomposition of shocks of
% reduced-form model;
% B. Nbig denotes the number of variables;
% C. VC_eps_chol denotes the Choleski decomposition of the
% variance-covariance matrix of shocks of reduced-form model;
% D. stdepsvec denotes the standard deviations of shocks of reduced-form
% model.
%
%
% NMC:     number of Monte Carlo simulations for the density of simulated
%           impulse responses
%
% Y:       Raw Data
%
% CIperc:  this is a 2 x 1 vector with lower and upper percentiles for 
%           the confidence itervals that are computed and, eventually,
%           plotted
%
%--------------------------------------------------------------------------
%
% OUTPUT:
%
% IRpoint: point estimate of impulse responses 
%
% IRup:    upper bound of impulse responses 
%
% IRlo:    lower bound of impulse responses
%
% IRmed:   median of impulse response distribution (at each horizon point)
%
%%----------------------------------------------------------------------------------------------------------------------------------
%% BACK OUT THE SIZE OF VARIOUS OBJECTS OF INTEREST
[Tbig,Nbig] = size(Y);
trendh = 1:Tbig;
Nbig   = length(VC_eps);
Nbigcomp = length(Bcomp);
vlag   = length(Bcomp)/Nbig;
BIG_X 		= zeros(Tbig-vlag,Nbig*vlag+1);
BIG_X(:,1) 	= ones(Tbig-vlag,1);
for lags = 1:1:vlag;
	lower_index_row		= vlag+1-lags;
	upper_index_row		= Tbig-lags;
	upper_index_col		= Nbig*lags+1;
	lower_index_col		= upper_index_col - Nbig + 1;
	BIG_X(:,lower_index_col:upper_index_col) = Y(lower_index_row:upper_index_row,:); 	
end;

BIG_Y = Y(vlag+1:Tbig,:);

istr  = sum(abs(dvec))>0;
iscon = sum(abs(cvec))>0;

allIRs 	 = zeros(IRhoriz+1, Nbig, Nbig, NMC);
IRup_1   = zeros(IRhoriz+1, Nbig, Nbig);
IRlo_1   = IRup_1;
IRup_2	 = IRup_1;
IRlo_2	 = IRup_1;
IRmed  	 = IRup_1;

IRpoint=VAR_irf(Bcomp, VC_eps, IRhoriz, IRtype, sizesho);

if vlag>1;
    cvec = [cvec; zeros((vlag-1)*Nbig, 1)];
    dvec = [dvec; zeros((vlag-1)*Nbig, 1)];
end

l_CI = length(CIperc);
%%--------------------------------------------------------------
Idmatcomp      	= eye(vlag*Nbig);
uncondmeancomp 	= inv(Idmatcomp-Bcomp)*cvec;
VC_epscompchol 	= zeros(Nbig*vlag, Nbig*vlag);
VC_epscompchol(1:Nbig, 1:Nbig) = chol(VC_eps)'; 
BIG_A 			= (BIG_X'*BIG_X)\BIG_X'*BIG_Y; 
mean			= BIG_A(1,:)';
vec_BIG_A 		= reshape(BIG_A,Nbig*(1+Nbig*vlag),1);
wishdof 		= Tbig;
scale			= Tbig-1;
COVAR 			= inv(BIG_X'*BIG_X);

%% DO MC RUNS
for mm=1:NMC;
	draw = mm;
	workbar(mm/NMC,'Performing Monte Carlo Integration...','Progress');
	%% HERE I EMPLOY ANTITHETIC SAMPLING WHICH SPEEDS UP THE CONVERGENCE
	if mod(mm,2) == 1
		% odd draw again
		VC_eps_draw 		= iwishrnd(scale*VC_eps,wishdof);
		vec_BIG_A_COVAR		= kron(VC_eps_draw,COVAR);
		vec_A_draw_u		= mvnrnd(zeros(Nbig*(1+Nbig*vlag),1)',vec_BIG_A_COVAR);
		vec_BIG_A_draw		= vec_BIG_A + vec_A_draw_u'; 
		BIG_A_draw			= reshape(vec_BIG_A_draw,1+Nbig*vlag,Nbig);
	else
		% even draw - take reflection
		vec_BIG_A_draw		= vec_BIG_A - vec_A_draw_u'; 
		BIG_A_draw			= reshape(vec_BIG_A_draw,1+Nbig*vlag,Nbig);
	end;
	%%
	Bcomp_draw			=[BIG_A_draw(2:end,:)'; kron(eye(vlag-1), eye(Nbig)), ... 
									zeros((vlag-1)*Nbig, Nbig)];
	allIRs(:, :, :, mm) = VAR_irf(Bcomp_draw, VC_eps_draw, IRhoriz, IRtype, sizesho);   
end

%% GET LOWER, MEDIAN AND UPPER VARIABLES OF INTEREST FROM SIMULATED DISTRIBUTION
%% IRFS
for hh=1:IRhoriz+1;
    for ii=1:Nbig;
        for jj=1:Nbig;
			IRpoint(hh,ii,jj) = IRpoint(hh,ii,jj);
            sliceloc   =  squeeze(allIRs(hh,ii,jj,:));
			if l_CI == 2;
            IRpercsloc = quantile(sliceloc,[CIperc(1);0.5;CIperc(2)]);
            IRlo_2(hh,ii,jj)   = IRpercsloc(1);
			IRmed(hh,ii,jj)    = IRpercsloc(2);
            IRup_2(hh,ii,jj)   = IRpercsloc(3);
			elseif l_CI == 4;
			IRpercsloc = quantile(sliceloc,[CIperc(1);CIperc(2);0.5;CIperc(3);CIperc(4)]);
            IRlo_1(hh,ii,jj)   = IRpercsloc(1);
			IRlo_2(hh,ii,jj)   = IRpercsloc(2);
            IRmed(hh,ii,jj)    = IRpercsloc(3);
			IRup_2(hh,ii,jj)   = IRpercsloc(4);
			IRup_1(hh,ii,jj)   = IRpercsloc(5);
			end;
        end
    end
end